package bus

import (
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"sku3dweb/log"
	"time"

	"sku3dweb/comm"

	"github.com/nats-io/nats.go"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func GetQrCode(req *model.OrderMsg) (map[string]interface{}, error) {

	var qrCode string
	err := NatsCenter.RequestPackApi(comm.CommPaySku3dCreateQrApi, req, &qrCode, &comm.RequestOptions{
		DeployPack: "comm",
	})
	if err != nil {
		log.Info("get error, timeout", err)
		return nil, err
	}

	return map[string]interface{}{
		"qrCode": qrCode,
	}, nil

}

func CloseOrder(req *model.OrderMsg) (*model.ResData, error) {

	var res model.ResData
	err := NatsCenter.Request("single.order.close", req, 3*time.Second, &res)
	if err != nil {
		log.Info("get error, timeout", err)
		return nil, err
	}

	return &res, nil
}

// 购买成功、失败的消息
func PayResult() *comm.NatsStreamWather {

	return &comm.NatsStreamWather{
		Stream:        "pay-sku3d-result",
		Topic:         "sku3d.payresult",
		Queue:         "sku3d-payresult-queue",
		AckWaitMinute: int64(1),
		Entity:        func() interface{} { return &model.ResData{} },
		Cb: func(msg *nats.Msg, entity interface{}) {
			defer func() {
				if err := recover(); err != nil {
					log.Info("系统异常", err)
				}
			}()

			session := &repo.RepoSession{Client: nil}

			fmt.Println("===============购买结果返回=================")
			log.Info("===============购买结果返回=================")
			data := entity.(*model.ResData)
			orderMsg := data.Data.(map[string]interface{})
			flag, _ := Paied(data.Code, orderMsg["_id"].(string), int(orderMsg["payMode"].(float64)), session)
			if flag {
				msg.Ack()
			}
		},
	}
}

func listenPaySucc(localBus *comm.NatsBus) {

	//支付成功消息
	localBus.PullPackMessage(comm.PackCommName, comm.CommPaySku3dSuccMsgSubject, 0.2, func(entity interface{}) (*comm.RespPackMessage, error) {
		fmt.Println("listenPaySucc==>back")
		defer func() {
			if err := recover(); err != nil {
				log.Info("pay succ 系统异常", err)
			}
		}()
		session := &repo.RepoSession{Client: nil}
		log.Info("===============购买成功结果返回=================")
		orderMsg := entity.(map[string]interface{})
		flag, _ := Paied(model.SUCCESS.Code, orderMsg["_id"].(string), int(orderMsg["payMode"].(float64)), session)
		if flag {
			return &comm.RespPackMessage{CanAck: true}, nil
		}
		return &comm.RespPackMessage{CanAck: false}, nil
	})

	//支付失败消息
	localBus.PullPackMessage(comm.PackCommName, comm.CommPaySku3dFailMsgSubject, 0.2, func(entity interface{}) (*comm.RespPackMessage, error) {
		fmt.Println("listenPaySucc==>back")
		defer func() {
			if err := recover(); err != nil {
				log.Info("pay succ 系统异常", err)
			}
		}()
		session := &repo.RepoSession{Client: nil}
		log.Info("===============购买失败结果返回=================")
		orderMsg := entity.(map[string]interface{})
		flag, _ := Paied(model.FAILD.Code, orderMsg["_id"].(string), int(orderMsg["payMode"].(float64)), session)
		if flag {
			return &comm.RespPackMessage{CanAck: true}, nil
		}
		return &comm.RespPackMessage{CanAck: false}, nil
	})
}

// 更新用户状态
func UpgradeUserType(uidStr string) (*model.ResData, error) {
	var res model.ResData
	err := NatsCenter.Request("user.upgrade.usertype", uidStr, 5*time.Second, &res)
	if err != nil {
		log.Info("get error, timeout", err)
		return nil, err
	}

	return &res, nil
}

// 支付完成后的逻辑
func Paied(code int, orderIdStr string, payMod int, session *repo.RepoSession) (bool, error) {
	log.Info("===============================购买完成后的逻辑start======================================")
	orderId, err := primitive.ObjectIDFromHex(orderIdStr)
	if err != nil {
		return false, err
	}

	// 1、查询出订单
	var order model.Order
	option := &repo.DocSearchOptions{
		CollectName: repo.CollectionOrder,
		Query:       repo.Map{"_id": orderId},
	}
	found, err := repo.RepoSeachDoc(session, option, &order)
	if !found {
		return true, err
	}

	// 订单成功失败都返回true
	if *order.Status == model.SuccessPay || *order.Status == model.FaildPay {
		return true, nil
	}

	// 订单支付失败则更新
	if code != model.SUCCESS.Code {
		// 更新订单状态为已关闭
		order.Status = &model.FaildPay
		order.PayMod = &payMod
		_, err = repo.RepoUpdateSetDoc(session, repo.CollectionOrder, orderIdStr, order)
		if err != nil {
			return false, err
		}
		return true, nil
	}

	// 1、用户中心更新为企业用户
	res, err := UpgradeUserType(order.UserId.Hex())
	if err != nil || res.Code != model.SUCCESS.Code {
		return false, err
	}

	// 2、更新订单状态
	order.Status = &model.SuccessPay
	order.PayMod = &payMod
	_, err = repo.RepoUpdateSetDoc(session, repo.CollectionOrder, orderIdStr, order)
	if err != nil {
		return false, err
	}

	// 3、插入(更新)到会员用户表
	// 3.1 查询会员用户表，判断是否存在
	var mu model.MemberUser
	option = &repo.DocSearchOptions{
		CollectName: repo.CollectionMemberUser,
		Query:       repo.Map{"parentId": order.UserId, "memberId": order.Member.Id},
	}
	found, _ = repo.RepoSeachDoc(session, option, &mu)
	fmt.Println(mu.Id)
	// 3.2 不存在插入，存在更新
	if !found {
		mu := &model.MemberUser{
			MemberId:    order.Member.Id,
			ParentId:    order.UserId,
			UserNum:     order.Member.UserNum,
			DeadTime:    time.Now().Add(time.Duration(*order.Member.Day) * time.Hour * 24),
			CompanyName: order.Member.CompanyName,
			Phone:       order.Member.Phone,
			CreateTime:  time.Now(),
			UpdateTime:  time.Now(),
		}
		_, err := repo.RepoAddDoc(session, repo.CollectionMemberUser, mu)
		if err != nil {
			return false, err
		}
	} else {
		// 1. 续费
		if order.Member.Day != nil {
			// 判断过期时间是否大于了现在，大于则加，小于则按当前时间
			if mu.DeadTime.After(time.Now()) {
				mu.DeadTime = mu.DeadTime.Add(time.Duration(*order.Member.Day) * time.Hour * 24)
			} else {
				mu.DeadTime = time.Now().Add(time.Duration(*order.Member.Day) * time.Hour * 24)
			}
			res, err := repo.RepoUpdateSetDoc(session, repo.CollectionMemberUser, mu.Id.Hex(), &mu)
			fmt.Println(res)
			if err != nil {
				return false, err
			}
		} else {
			// 2.添加席位
			num := *mu.UserNum + *order.Member.UserNum
			mu.UserNum = &num
			res, err := repo.RepoUpdateSetDoc(session, repo.CollectionMemberUser, mu.Id.Hex(), &mu)
			fmt.Println(res)
			if err != nil {
				return false, err
			}
		}
	}

	fmt.Println("===============================购买完成后的逻辑end======================================")
	return true, nil
}
